#!/bin/bash
function m_replace () {
#
# replaces all occurrencies of a string in a file with another string
#
if [[  $# != 2 ]]
then
	echo "Error!!! Please specify two arguments!
Description:
	Replaces all occurrencies of an \"oldstring\" in a file with a \"newstring\" string
Usage: 
	m_replace \"oldstring\" \"newstring\" < oldfile > newfile"
	return 1
else
	sed "s/$1/$2/g"  
fi
}

function m_distance (){
#
# computes distance between two vectors
#
if [[  $# != 6 ]]
then
	echo "Error!!! Please specify six argumens!
Description:
	Computes the distance between two points
Usage: 
	m_distance x1 y1 z1 x2 y2 z2"
else
#error?????????
#	dx2=`echo \( $1 \-  $4 \) \* \( $1 \-  $4 \)  | bc -l `
#	dy2=`echo \( $2 \-  $5 \) \* \( $2 \-  $5 \)  | bc -l `
#	dz2=`echo \( $3 \-  $6 \) \* \( $3 \-  $6 \)  | bc -l `
	dx2=`echo "$1 -  $4"  | bc -l `
	dy2=`echo "$2 -  $5"  | bc -l `
	dz2=`echo "$3 -  $6"  | bc -l `
	dist=`awk -v dx2=$dx2 -v dy2=$dy2 -v dz2=$dz2 'BEGIN { printf ("%4.3f",sqrt(dx2*dx2+dy2*dy2+dz2*dz2))}' < /dev/null `
	echo $dist
fi
}

function m_sum (){
#
# Returns the sum of two numbers
#
if [[  $# != 2 ]]
then
	echo "Error!!! Please specify two arguments
Description:
	Returns the sum of two numbers
Usage: 
	m_sum num1 num2"
else
	sum=` echo $1 + $2 | bc -l `
echo $sum
fi
}

function m_multiply (){
#
# Returns the product of two numbers
#
if [[  $# != 2 ]]
then
	echo "Error!!! Please specify two arguments
Description:
	Returns the product of two numbers
Usage: 
	m_multiply num1 num2"
else
	prod=` echo $1 \* $2 | bc -l `
echo $prod
fi
}

function m_divide (){
#
# Returns the ratio of two numbers
#
if [[  $# != 2 ]]
then
	echo "Error!!! Please specify two arguments
Description:
	Returns the ratio of two numbers
Usage: 
	m_ratio num1 num2"
else
	ratio=` echo $1 \/ $2 | bc -l `
echo $ratio
fi
}



function m_list (){
#
# lists between two numbers
#
if [[  $# != 2 ]]
then
	echo "Error!!!
	 Please specify two arguments
Description:
	Creates an ordered list of numbers between two integers
Usage: 
	m_list nmin nmax"
else
	eval echo {$1..$2}
fi
}

function m_change () {
#
# adds a delta times an integer to a central value
#
if [[  $# != 3 ]]
then
	echo "Error!!!  
	Please specify three arguments
Description:
	Adds a delta times an integer to a central value
Usage: 
	m_change x0 n delta"
else
	aa=`echo $1 + \( $2 \* $3 \) | bc -l `
echo $aa
fi
}


function m_getcolumn () {
#
# greps a string from a file and gets the last instance; then takes the nth column
#
if [[  $# != 2 ]]
then
echo "Error!!! 
	Please specify two arguments
Description:
	Greps a string from a file and gets the last instance; then takes the nth column
Usage: 
	m_getcolumn \"STRING\" column < file"
else
grep $1  | awk -v c=$2 '{print $c}'  | tail -1
fi
}


function m_norm () {
#
# norm of a vector
#
if [[  $# != 3 ]]
	then
	echo "Error!!!  
	Please specify three arguments
Description:
	Returns the norm of a vector
Usage:
	m_norm x y z
"
else
	echo sqrt \( $1 \* $1 + $2 \* $2 + $3 \* $3  \) | bc -l 
fi
}

function m_vecprod () {
#
# vec1 X vec2
#
if [[  $# != 6 ]]
	then
	echo "Error!!!  
	Please specify six arguments
Description:
	Returns vector product of two vectors
Usage: 
	m_vecprod x1 y1 z1 x2 y2 z2"
else
	x1=$1
	y1=$2
	z1=$3
	x2=$4
	y2=$5
	z2=$6
	x3=`echo $y1 \* $z2 - $z1 \* $y2 | bc -l `
	y3=`echo - $x1 \* $z2 + $z1 \* $x2 | bc -l `
	z3=`echo $x1 \* $y2 - $y1 \* $x2 | bc -l `
	echo $x3 $y3 $z3 
fi
}

function m_unitvec () {
#
# unit vector
#
if [[  $# != 3 ]]
	then
	echo "Error!!!  
	Please specify three arguments
Description:
	Returns a normalized input vector
Usage: 
	m_unitvec x y z "
else
	nor=`m_norm $1 $2 $3`
	xn=` echo $1 / $nor | bc -l `
	yn=` echo $2 / $nor | bc -l `
	zn=` echo $3 / $nor | bc -l `
	echo $xn $yn $zn 
fi
}

function m_scalar () {
#
# vec1 scalar vec2
#
if [[  $# != 6 ]]
then
	echo "Error!!!  
	Please specify six arguments
Description:
	Returns scalar product of two vectos
Usage: 
	m_scalar x1 y1 z1 x2 y2 z2"
else
	x3=`echo $1 \* $4  | bc -l `
	y3=`echo $2 \* $5  | bc -l `
	z3=`echo $3 \* $6  | bc -l `
	sp=`echo $x3 + $y3 + $z3 | bc -l `
	echo $sp
fi
}

function m_pdbvecjoin () {
#
# vector joining two atoms in a pdb file
#
if [[  $# != 3 ]]
then
	echo "Error!!!  
	Please specify three arguments
Description:
	Returns a vector joining  two atoms in a pdb file
Usage: 
	m_pdbvecjoin at1 at2 pdbfile"
else
	awk -v at1=$1 -v at2=$2 ' ($2==at1 || $2==at2) {print $0}
	' < $3 | cut -c 31-54 | awk '
	(NR==1){x1=$1;y1=$2;z1=$3}
	(NR==2){x2=$1;y2=$2;z2=$3}
	END {print x2-x1,y2-y1,z2-z1}' 
fi
}


function m_atan2 () {
#
# atan2 (x,y)
#
if [[  $# != 2 ]]
then
	echo "Error!!!  
	Please specify two arguments
Description:
	Returns the angle in degrees between the positive X-axis of a plane and the point given by the coordinates (x,y) on it:

\        
 \        
  * (x,y)      
   \     
    \    
     \_______    
      \      |   
       \angle|
      --\---------------------->
         \                     X
          \

Usage: 
	m_myatan2 x y"
else
	awk -v x=$1 -v y=$2 'BEGIN {print atan2(x,y)*180./3.1415926}' < /dev/null
fi
}


function m_pdbtorsion () {
#
# torsion of 4 atoms in file pdb
#
if [[  $# != 5 ]]
then
	echo "Error!!!  Please specify five arguments
Description:
	Returns torsion defined by 4 atoms in a pdb file
                           Atom4
                            /
             <-.           / 
           angle\         /
     Atom2-------|-----Atom3  
      /      \__/
     /       
    /
 Atom1

Usage:
	m_pdbtorsion n1 n2 n3 n4 file"
else
	file=$5
	b1=`m_pdbvecjoin $1 $2 $file`
	b2=`m_pdbvecjoin $2 $3 $file`
	b3=`m_pdbvecjoin $3 $4 $file`
	b12=`m_vecprod $b1 $b2 `
	b23=`m_vecprod $b2 $b3 `
	n1=`m_unitvec $b12`
	n2=`m_unitvec $b23`
	x=`m_scalar $n1 $n2 `
	nb2=`m_unitvec $b2`
	m1=`m_vecprod $n1 $nb2`
	y=`m_scalar $m1 $n2 `
	dih=`m_myatan2 $y $x `
	echo \- $dih | bc -l
fi
}


function m_interpolate () {
#
# interpolation of two numbers with n+1 points (including the extremes) 
#
if [[  $# != 3 ]]
then
	echo "Error!!!  
	Please specify three arguments
Description:
	Returns a list of real numbers that interpolates two given extremes A and B with n+1 points (from A=0 to B=n)
Usage:
	m_interpolate xa xb n"
else
	xa=$1
	xb=$2
	n=$3
	delta=`echo \( $xb \- $xa \) \/ $n | bc -l`
	for i in `m_list 0 $n` 
        do
 	       a[i]=`echo $xa + $i \* $delta  | bc -l | awk '{printf ("%10.4f",$1)}'`
        done
	echo ${a[@]}
fi
}
